/*******************************************************************
  Interfacing PIC18F46K22 microcontroller with ST7735S TFT display.
  Graphics test example.
  C Code for mikroC PRO for PIC compiler.
  Internal oscillator used @ 16MHz
  Configuration words: CONFIG1H = 0x0028
                       CONFIG2L = 0x0018
                       CONFIG2H = 0x003C
                       CONFIG3H = 0x0037
                       CONFIG4L = 0x0081
                       CONFIG5L = 0x000F
                       CONFIG5H = 0x00C0
                       CONFIG6L = 0x000F
                       CONFIG6H = 0x00E0
                       CONFIG7L = 0x000F
                       CONFIG7H = 0x0040
  This is a free software with NO WARRANTY.
  https://simple-circuit.com/

*********************************************************************
  This is a library for several Adafruit displays based on ST77* drivers.

  Works with the Adafruit 1.8" TFT Breakout w/SD card
    ----> http://www.adafruit.com/products/358
  The 1.8" TFT shield
    ----> https://www.adafruit.com/product/802
  The 1.44" TFT breakout
    ----> https://www.adafruit.com/product/2088
  as well as Adafruit raw 1.8" TFT display
    ----> http://www.adafruit.com/products/618

  Check out the links above for our tutorials and wiring diagrams.
  These displays use SPI to communicate, 4 or 5 pins are required to
  interface (RST is optional).

  Adafruit invests time and resources providing this open source code,
  please support Adafruit and open-source hardware by purchasing
  products from Adafruit!

  Written by Limor Fried/Ladyada for Adafruit Industries.
  MIT license, all text above must be included in any redistribution
****************************************************************************/

// define ST7735 TFT module pin connections
//   RST (optional), CS and DC
#define TFT_RST       RD0_bit
#define TFT_CS        RD1_bit
#define TFT_DC        RD2_bit
#define TFT_RST_DIR   TRISD0_bit
#define TFT_CS_DIR    TRISD1_bit
#define TFT_DC_DIR    TRISD2_bit

#include "ST7735.c"       // include ST7735 TFT display driver source code
#include "GFX_Library.c"  // include graphics library source code

void testlines(uint16_t color) {
  uint8_t x, y;
  display_fillScreen(ST7735_BLACK);
  for (x=0; x < display_width; x+=6) {
    display_drawLine(0, 0, x, display_height-1, color);
  }
  for (y=0; y < display_height; y+=6) {
    display_drawLine(0, 0, display_width-1, y, color);
  }

  display_fillScreen(ST7735_BLACK);
  for (x=0; x < display_width; x+=6) {
    display_drawLine(display_width-1, 0, x, display_height-1, color);
  }
  for (y=0; y < display_height; y+=6) {
    display_drawLine(display_width-1, 0, 0, y, color);
  }

  display_fillScreen(ST7735_BLACK);
  for (x=0; x < display_width; x+=6) {
    display_drawLine(0, display_height-1, x, 0, color);
  }
  for (y=0; y < display_height; y+=6) {
    display_drawLine(0, display_height-1, display_width-1, y, color);
  }

  display_fillScreen(ST7735_BLACK);
  for (x=0; x < display_width; x+=6) {
    display_drawLine(display_width-1, display_height-1, x, 0, color);
  }
  for (y=0; y < display_height; y+=6) {
    display_drawLine(display_width-1, display_height-1, 0, y, color);
  }
}

void testdrawtext(char *text, uint16_t color) {
  display_setCursor(0, 0);
  display_setTextColor(color, color);
  display_setTextWrap(true);
  display_puts(text);
}

void testfastlines(uint16_t color1, uint16_t color2) {
  uint8_t x, y;
  display_fillScreen(ST7735_BLACK);
  for (y=0; y < display_height; y+=5) {
    display_drawHLine(0, y, display_width, color1);
  }
  for (x=0; x < display_width; x+=5) {
    display_drawVLine(x, 0, display_height, color2);
  }
}

void testdrawrects(uint16_t color) {
  uint8_t x;
  display_fillScreen(ST7735_BLACK);
  for (x=0; x < display_width; x+=6) {
    display_drawRect(display_width/2 -x/2, display_height/2 -x/2 , x, x, color);
  }
}

void testfillrects(uint16_t color1, uint16_t color2) {
  uint8_t x;
  display_fillScreen(ST7735_BLACK);
  for (x=display_width-1; x > 6; x-=6) {
    display_fillRect(display_width/2 -x/2, display_height/2 -x/2 , x, x, color1);
    display_drawRect(display_width/2 -x/2, display_height/2 -x/2 , x, x, color2);
  }
}

void testfillcircles(uint8_t radius, uint16_t color) {
  uint8_t x, y;
  for (x=radius; x < display_width; x+=radius*2) {
    for (y=radius; y < display_height; y+=radius*2) {
      display_fillCircle(x, y, radius, color);
    }
  }
}

void testdrawcircles(uint8_t radius, uint16_t color) {
  uint8_t x, y;
  for (x=0; x < display_width+radius; x+=radius*2) {
    for (y=0; y < display_height+radius; y+=radius*2) {
      display_drawCircle(x, y, radius, color);
    }
  }
}

void testtriangles() {
  uint16_t color = 0xF800;
  uint8_t t,
          w = display_width / 2,
          x = display_height - 1,
          y = 0,
          z = display_width;
  display_fillScreen(ST7735_BLACK);
  for(t = 0 ; t <= 15; t++) {
    display_drawTriangle(w, y, y, x, z, x, color);
    x-=4;
    y+=4;
    z-=4;
    color+=100;
  }
}

void testroundrects() {
  uint16_t color = 100;
  uint8_t i, t;
  display_fillScreen(ST7735_BLACK);
  for(t = 0 ; t <= 4; t+=1) {
    uint8_t x = 0,
            y = 0,
            w = display_width - 2,
            h = display_height - 2;
    for(i = 0 ; i <= 16; i+=1) {
      display_drawRoundRect(x, y, w, h, 5, color);
      x+=2;
      y+=3;
      w-=4;
      h-=6;
      color+=1100;
    }
    color+=100;
  }
}

uint16_t millis = 0;
void Interrupt() {
  TMR0IF_bit = 0;
  millis++;
}

void tftPrintTest() {
  float p = 3.1415926;
  display_setTextWrap(false);
  display_fillScreen(ST7735_BLACK);
  display_setCursor(0, 30);
  display_setTextColor(ST7735_RED, ST7735_BLACK);
  display_setTextSize(1);
  display_puts("Hello World!\r\n");
  display_setTextColor(ST7735_YELLOW, ST7735_YELLOW);
  display_setTextSize(2);
  display_puts("Hello World!\r\n");
  display_setTextColor(ST7735_GREEN, ST7735_GREEN);
  display_setTextSize(3);
  display_puts("Hello World!\r\n");
  display_setTextColor(ST7735_BLUE, ST7735_BLUE);
  display_setTextSize(4);
  display_printf("%f", 1234.567);
  delay_ms(1500);
  display_setCursor(0, 0);
  display_fillScreen(ST7735_BLACK);
  display_setTextColor(ST7735_WHITE, ST7735_WHITE);
  display_setTextSize(0);
  display_puts("Hello World!\r\n");
  display_setTextSize(1);
  display_setTextColor(ST7735_GREEN, ST7735_GREEN);
  display_printf("%8.6f", p);
  display_puts(" Want pi?\r\n");
  display_puts(" \r\n");
  display_printf("%6LX", 8675309); // print 8,675,309 out in HEX!
  display_puts(" Print HEX!\r\n");
  display_puts(" \r\n");
  display_setTextColor(ST7735_WHITE, ST7735_WHITE);
  display_puts("Sketch has been\r\n");
  display_puts("running for: \r\n");
  display_setTextColor(ST7735_MAGENTA, ST7735_MAGENTA);
  display_printf("%u", millis/1024);
  display_setTextColor(ST7735_WHITE, ST7735_WHITE);
  display_puts(" seconds.");
  TMR0ON_bit = 0;   // turn Timer0 off
}

void mediabuttons() {
  // play
  display_fillScreen(ST7735_BLACK);
  display_fillRoundRect(25, 10, 78, 60, 8, ST7735_WHITE);
  display_fillTriangle(42, 20, 42, 60, 90, 40, ST7735_RED);
  delay_ms(500);
  // pause
  display_fillRoundRect(25, 90, 78, 60, 8, ST7735_WHITE);
  display_fillRoundRect(39, 98, 20, 45, 5, ST7735_GREEN);
  display_fillRoundRect(69, 98, 20, 45, 5, ST7735_GREEN);
  delay_ms(500);
  // play color
  display_fillTriangle(42, 20, 42, 60, 90, 40, ST7735_BLUE);
  delay_ms(50);
  // pause color
  display_fillRoundRect(39, 98, 20, 45, 5, ST7735_RED);
  display_fillRoundRect(69, 98, 20, 45, 5, ST7735_RED);
  // play color
  display_fillTriangle(42, 20, 42, 60, 90, 40, ST7735_GREEN);
}

// main function
void main()
{
  OSCCON = 0x70;   // set internal oscillator to 16MHz
  ANSELC = 0;      // configure all PORTC pins as digital
  ANSELD = 0;      // configure all PORTD pins as digital
  // Timer0 Interrupt configuration
  TMR0L      = 0;      // reset Timer0 low register
  T0CON      = 0xC3;   // use Timer0 as 8-bit timer with prescaler = 16
  GIE_bit    = 1;      // enable global interrupts
  TMR0IE_bit = 1;      // enable Timer0 interrupt

  SPI1_Init();     // initialize the SPI1 module with default settings
  // Use this initializer if using a 1.8" TFT screen:
  tft_initR(INITR_BLACKTAB);      // Init ST7735S chip, black tab

  // OR use this initializer (uncomment) if using a 1.44" TFT:
  //tft_initR(INITR_144GREENTAB); // Init ST7735R chip, green tab

  // OR use this initializer (uncomment) if using a 0.96" 180x60 TFT:
  //tft_initR(INITR_MINI160x80);  // Init ST7735S mini display

  // large block of text
  display_fillScreen(ST7735_BLACK);
  testdrawtext((char *)"Lorem ipsum dolor sit amet, consectetur adipiscing elit. Curabitur adipiscing ante sed nibh tincidunt feugiat. Maecenas enim massa, fringilla sed malesuada et, malesuada sit amet turpis. Sed porttitor neque ut ante pretium vitae malesuada nunc bibendum. Nullam aliquet ultrices massa eu hendrerit. Ut sed nisi lorem. In vestibulum purus a tortor imperdiet posuere. ", ST7735_WHITE);
  delay_ms(1000);

  // tft print function!
  tftPrintTest();
  delay_ms(4000);

  // a single pixel
  display_drawPixel(display_width/2, display_height/2, ST7735_GREEN);
  delay_ms(500);

  // line draw test
  testlines(ST7735_YELLOW);
  delay_ms(500);

  // optimized lines
  testfastlines(ST7735_RED, ST7735_BLUE);
  delay_ms(500);

  testdrawrects(ST7735_GREEN);
  delay_ms(500);

  testfillrects(ST7735_YELLOW, ST7735_MAGENTA);
  delay_ms(500);

  display_fillScreen(ST7735_BLACK);
  testfillcircles(10, ST7735_BLUE);
  testdrawcircles(10, ST7735_WHITE);
  delay_ms(500);

  testroundrects();
  delay_ms(500);

  testtriangles();
  delay_ms(500);

  mediabuttons();
  delay_ms(500);

  while(1)
  {
    display_invert(1);
    delay_ms(500);
    display_invert(0);
    delay_ms(500);
  }
  
}

// end of code.